/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.sensor;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObjectExistsException;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.MultiFileLoader;
import org.openide.util.NbBundle;
import org.openide.util.NbPreferences;

/**
 * A base DataObject for SensorData When constructed a background task is
 * started to process the content into a displayable image.
 *
 * @author Timon Veenstra
 */
public abstract class SensorDataObject extends MultiDataObject {

    private SensorPalette palette;
    private SensorGrid grid;
    public static final String PROP_PALETTE_CHANGED = "paletteCahnged";
    private static final String PALETTE_PROPERTY = "property";
    private static final String WARING_SAVE_PALETTE_MESSAGE = NbBundle.getMessage(SensorDataObject.class, "sensor.palette.warning.save.palette");
    
    private static final AppLogger LOG = AppLogFactory.getLogger(SensorDataObject.class);

    public SensorDataObject(FileObject fo, MultiFileLoader loader) throws DataObjectExistsException {
        super(fo, loader);
    }

    public SensorGrid getSensorGrid() {
        if (grid == null) {
            grid = new SensorGrid(getProcessor());
        }
        return grid;

    }

    protected abstract SensorProcessor getProcessor();

    public SensorPalette getPalette() {
        if (palette == null) {
            ObjectInputStream inputStream = null;
            try {
                byte[] bytes = NbPreferences.forModule(this.getClass()).getByteArray(PALETTE_PROPERTY, new byte[0]);
                if (bytes.length > 0) {
                    ByteArrayInputStream iStream = new ByteArrayInputStream(bytes);
                    inputStream = new ObjectInputStream(iStream);
                    palette = (SensorPalette) inputStream.readObject();
                }
                else{
                     palette = SensorPalette.PALLET00_10;
                }
            } catch (ClassNotFoundException ex) {
                palette = SensorPalette.PALLET00_10;
            } catch (IOException ex) {
                palette = SensorPalette.PALLET00_10;
            } finally {
                try {
                    if (inputStream != null) {
                        inputStream.close();
                    }
                } catch (IOException ex) {
                    //Do nothing
                }
            }
        }
        return palette;
    }

    public void setPalette(SensorPalette palette) {
        SensorPalette old = this.palette;
        this.palette = palette;
        this.firePropertyChange(PROP_PALETTE_CHANGED, old, palette);
        ByteArrayOutputStream bStream = new ByteArrayOutputStream();
        ObjectOutputStream outputStream;
        try {
            outputStream = new ObjectOutputStream(bStream);
            outputStream.writeObject(palette);
            byte[] byteVal = bStream.toByteArray();
            NbPreferences.forModule(this.getClass()).putByteArray(PALETTE_PROPERTY, byteVal);
        } catch (IOException ex) {
            LOG.warning(WARING_SAVE_PALETTE_MESSAGE);
        }
    }
}
