/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.sensor.project;

import java.awt.Dialog;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.io.SyncFailedException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import javax.swing.AbstractAction;
import javax.swing.filechooser.FileFilter;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.sensor.SensorPreference;
import nl.cloudfarming.client.sensor.SensorProcessorProvider;
import nl.cloudfarming.client.util.FilePickerUtil;
import org.openide.DialogDescriptor;
import org.openide.DialogDisplayer;
import org.openide.awt.ActionID;
import org.openide.awt.ActionReference;
import org.openide.awt.ActionReferences;
import org.openide.awt.ActionRegistration;
import org.openide.filesystems.FileChooserBuilder;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.util.Lookup;
import org.openide.util.LookupEvent;
import org.openide.util.LookupListener;
import org.openide.util.NbBundle;
import org.openide.util.NbBundle.Messages;

/*
 * Filepicker to copy sensorDataFiles into sensor project @Author: Merijn
 * Zengers
 */
@ActionID(category = "Project",
id = "nl.cloudfarming.client.sensor.project.ImportSensorFiles")
@ActionRegistration(iconBase = "nl/cloudfarming/client/sensor/project/loadsensor.png",
displayName = "#CTL_ImportSensorFiles")
@ActionReferences({
    @ActionReference(path = "Menu/File", position = 0)
})
@Messages({
    "sensor.filepicker.title=Choose a sensor file",
    "sensor.filepicker.approve=Choose",
    "sensor.filepicker.dialog.no_sensor_modules_installed.title=Cannot import sensor files.",
    "sensor.filepicker.dialog.no_sensor_modules_installed.text=Cannot import sensor files, there is no sensor module installed.",
    "sensor.filepicker.io_exception_message=A IO error occurred",
    "sensor.filepicker.file_could_not_be_processed=The file {0} could not be processed!",
    "CTL_ImportSensorFiles=Import sensor files",
    "sensor.filepicker.all_supported_filetypes.text=All supported sensor types"
})
public final class ImportSensorFiles extends AbstractAction implements ActionListener {

    private static final String FILEPICKER_TITLE = NbBundle.getMessage(ImportSensorFiles.class, "sensor.filepicker.title");
    private static final String FILEPICKER_APPROVE = NbBundle.getMessage(ImportSensorFiles.class, "sensor.filepicker.approve");
    private static final String NO_SENSOR_MODULES_INSTALLED_TITLE = NbBundle.getMessage(ImportSensorFiles.class, "sensor.filepicker.dialog.no_sensor_modules_installed.title");
    private static final String NO_SENSOR_MODULES_INSTALLED_TEXT = NbBundle.getMessage(ImportSensorFiles.class, "sensor.filepicker.dialog.no_sensor_modules_installed.text");
    private static final String FILEPICKER_ALL_SUPPORTED_FILETYPES_TEXT = NbBundle.getMessage(ImportSensorFiles.class, "sensor.filepicker.all_supported_filetypes.text");
    private static final String IO_EXCEPTION_MESSAGE = NbBundle.getMessage(ImportSensorFiles.class, "sensor.filepicker.io_exception_message");
    private final Lookup.Result<SensorProcessorProvider> sensorProcessorProviderResult = Lookup.getDefault().lookupResult(SensorProcessorProvider.class);
    private final ProxyFileFilter filterAllSensors = new ProxyFileFilter();
    private SensorProject context = null;
    private Dialog dialog = null;
    private final List<FileObject> copyQueue = Collections.synchronizedList(new ArrayList<FileObject>());
    private static final AppLogger LOG = AppLogFactory.getLogger(ImportSensorFiles.class);

    public ImportSensorFiles(SensorProject context) {
        super(NbBundle.getMessage(ImportSensorFiles.class, "CTL_ImportSensorFiles"));
        this.context = context;

        for (SensorProcessorProvider processorProvider : sensorProcessorProviderResult.allInstances()) {
            filterAllSensors.addFilter(FilePickerUtil.createSwingFileFilter(processorProvider.getSupportedExtentions(), processorProvider.getFormatDescription()));
        }

        sensorProcessorProviderResult.addLookupListener(new LookupListener() {

            @Override
            public void resultChanged(LookupEvent ev) {
                synchronized (filterAllSensors) {
                    filterAllSensors.reset();
                    for (SensorProcessorProvider processorProvider : ((Lookup.Result<SensorProcessorProvider>) ev.getSource()).allInstances()) {
                        filterAllSensors.addFilter(FilePickerUtil.createSwingFileFilter(processorProvider.getSupportedExtentions(), processorProvider.getFormatDescription()));
                    }
                }
            }
        });
    }

    @Override
    public void actionPerformed(ActionEvent ev) {
        File workingDirectory = new File(SensorPreference.LAST_DIR.getValue());
        FileChooserBuilder fcb = new FileChooserBuilder("Sensor-directory");
        fcb.setTitle(FILEPICKER_TITLE);
        fcb.setDefaultWorkingDirectory(workingDirectory);
        fcb.setApproveText(FILEPICKER_APPROVE);

        // Here we loop through all ProcessorProviders to retreive all possible extensions
        // also save them to supportedExtensions to build an "All sensor filetypes" filefilter from
        final ArrayList<String> supportedExtensions = new ArrayList<>();
        for (SensorProcessorProvider processorProvider : Lookup.getDefault().lookupAll(SensorProcessorProvider.class)) {
            fcb.addFileFilter(FilePickerUtil.createSwingFileFilter(processorProvider.getSupportedExtentions(), processorProvider.getFormatDescription()));
            supportedExtensions.addAll(processorProvider.getSupportedExtentions());
        }

        if (supportedExtensions.isEmpty()) {
            final DialogDescriptor descriptor = new DialogDescriptor(NO_SENSOR_MODULES_INSTALLED_TEXT, NO_SENSOR_MODULES_INSTALLED_TITLE, true,
                    DialogDescriptor.OK_CANCEL_OPTION, DialogDescriptor.OK_OPTION,
                    new ActionListener() {

                        @Override
                        public void actionPerformed(ActionEvent e) {
                            dialog.dispose();
                        }
                    });

            dialog = DialogDisplayer.getDefault().createDialog(descriptor);
            dialog.setVisible(true);
        } else {

            // Create the "All sensor filetypes" filefilter
            FileFilter fileFilter = FilePickerUtil.createSwingFileFilter(supportedExtensions, FilePickerUtil.getFileFilterDescription(FILEPICKER_ALL_SUPPORTED_FILETYPES_TEXT, supportedExtensions));
            fcb.addFileFilter(fileFilter);
            // Open file pick dialog, retreive selected sensorfile
            File selectedFile = fcb.showOpenDialog();

            FileObject destination = context.getSensorFolder(false);

            if (selectedFile != null) {
                if (selectedFile.isDirectory()) {
                    SensorPreference.LAST_DIR.saveValue(selectedFile.getPath());
                    synchronized (filterAllSensors) {
                        List<File> selectedFiles = Arrays.asList(selectedFile.listFiles(filterAllSensors));
                        for (File file : selectedFiles) {
                            if (fileFilter.accept(file)) {
                                FileObject fileObject = FileUtil.toFileObject(file);
                                copyQueue.add(fileObject);
                            }
                        }
                    }
                } else if (selectedFile.isFile()) {
                    SensorPreference.LAST_DIR.saveValue(selectedFile.getParentFile().getPath());
                    copyQueue.add(FileUtil.toFileObject(selectedFile));
                }
                // start copy process
                try {
                    copy(copyQueue, destination);
                } catch (IOException ex) {
                    LOG.error(IO_EXCEPTION_MESSAGE, ex);
                }
            }
        }
    }

    // TODO: Util method should be in util?
    /**
     * Copy queue to sensor project dir.
     *
     * @param copyQueue List of files to copy to the archive dir.
     * @Param destination FileObject to copy the files to.
     * @throws IOException
     */
    protected void copy(List<FileObject> copyQueue, FileObject destination) throws IOException {

        for (FileObject file : copyQueue) {
            try {
                FileUtil.copyFile(file, destination, file.getName());
            } catch (SyncFailedException ex) {
                //Do nothing file already exists
            }

        }
        copyQueue.clear();
    }

    private class ProxyFileFilter implements java.io.FileFilter {

        private final List<FileFilter> filters = new ArrayList<>();

        @Override
        public boolean accept(File pathname) {
            for (FileFilter filter : filters) {
                if (filter.accept(pathname)) {
                    return true;
                }
            }
            return false;
        }

        public void reset() {
            filters.clear();
        }

        public void addFilter(FileFilter filter) {
            filters.add(filter);
        }
    }
}
